import { TransformConfig } from "../internal";
export declare type Disposer = () => void;
export declare type Handler<T = unknown> = (value: T) => void;
export declare type Selector<T = unknown, X = unknown> = (base: T) => X;
export declare type Producer<T = unknown> = (draft: T) => (void | T);
export declare type Updater<T = unknown> = Producer<T> | Partial<T> | T;
export declare type Transformer<T, R> = (lens: Lens<T>) => R;
export interface Lens<T = unknown> {
    value(): T;
    subscribe(handler: Handler<T>): Disposer;
    update(producer: Updater<T>): void;
    /**
     * transform produces a new lens from a single other lens, given a config.
     * Typically used by transformers
     */
    transform<R>(config: Partial<TransformConfig<T, R>>): Lens<R>;
    do<R>(transformer: Transformer<T, R>): R;
    do<A, R>(transformer: Transformer<T, Lens<A>>, transformer2: Transformer<A, R>): R;
    do<A, B, R>(t1: Transformer<T, Lens<A>>, t2: Transformer<A, Lens<B>>, t3: Transformer<B, R>): R;
    do<A, B, C, R>(t1: Transformer<T, Lens<A>>, t2: Transformer<A, Lens<B>>, t3: Transformer<B, Lens<C>>, t4: Transformer<C, R>): R;
    do<A, B, C, D, R>(t1: Transformer<T, Lens<A>>, t2: Transformer<A, Lens<B>>, t3: Transformer<B, Lens<C>>, t4: Transformer<C, Lens<D>>, t5: Transformer<D, R>): R;
    do(...transformers: (Transformer<any, any>)[]): any;
    select<K extends keyof T>(selector: K): KeyedLens<T[K], K>;
    select<K1 extends keyof T, K2 extends keyof T[K1]>(selector: K1, selector2: K2): KeyedLens<T[K1][K2], K2>;
    select<K1 extends keyof T, K2 extends keyof T[K1], K3 extends keyof T[K1][K2]>(selector: K1, selector2: K2, selector3: K3): KeyedLens<T[K1][K2][K3], K3>;
    select<K1 extends keyof T, K2 extends keyof T[K1], K3 extends keyof T[K1][K2], K4 extends keyof T[K1][K2][K3]>(selector: K1, selector2: K2, selector3: K3, selector4: K4): KeyedLens<T[K1][K2][K3][K4], K4>;
    select<K1 extends keyof T, K2 extends keyof T[K1], K3 extends keyof T[K1][K2], K4 extends keyof T[K1][K2][K3], K5 extends keyof T[K1][K2][K3][K4]>(selector: K1, selector2: K2, selector3: K3, selector4: K4, selector5: K5): KeyedLens<T[K1][K2][K3][K4][K5], K5>;
}
export interface KeyedLens<T, K = keyof T> extends Lens<T> {
    key: K;
}
export declare function isLens(thing: any): thing is Lens;
